/*****************************************************************************\
 *
 * geometryClass.h
 *
 * Updated: 14.12.2005
 * 
 * geometryClass  - class for create objects geometry
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef _geometryClass_h_
#define _geometryClass_h_

#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoSeparator.h>

#include "lib3ds.h"
#include "common.h"
#include "smoothGroupsClass.h"
#include "sceneClass.h"

// maximal number of smoothing groups that can be used
#define MAX_SMOOTH_GROUPS 33

/*
 * Class for computing and creating object geometry, augument vertices about 
 * smooth groups information and performing stepst to put object together
 * (add vertices, indexed face set and computing normals with respect to smoothg groups)
 */
class geometryClass : sceneClass {

  smoothGroupsClass   *smGroup;

  SoSeparator         *meshSet;
  SoSeparator         *materialSet;

  // add vertices to scene graph
  void addCoordinate();
  // add trinagles definition to scene graph
  void addIndexedFaceSet(int *faces, SoSeparator *smg, int csmgr);
  
public:
  
   // create part of scene graph that stand for geometry
  void createGeometry(int mat_id, int faceted);
  
  // actualize separtor for new meshmat part
  void initBit(SoSeparator *);

  meshLib3 *getMesh();
  smoothGroupsClass *getSmGroup();

  geometryClass(sceneClass *);
  ~geometryClass();
};
#endif
