/*****************************************************************************\
 *
 * keyFrameClass.cpp
 *
 * Updated: 14.12.2005
 *
 * keyFrameClass  - perform mesh's transformations
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "keyFrameClass.h"


/*
 * This function solve hierarchical structure in scene. It search for root node for actual object and apply
 * needed transformation.
 */
void keyFrameClass::findTopParent(int *clockshape, SbVec3f *p_pos, SbVec4f *p_rot, SbVec3f *p_scl, kfmeshLib3 *kfmesh) {
  kfmeshLib3		*kfmesh2 = NULL;
 
  if( kfmesh->parent[0] != '\0' )
  {    
    kfmodel->getKeyFrame(kfmesh->parent, &kfmesh2);
	  findTopParent(clockshape, p_pos, p_rot, p_scl, kfmesh2);
    


    // we creating separately matrix for separately paretn's transformations
    
    // parent translation matrix
    SbMatrix posMatrix;
    posMatrix.setTranslate(SbVec3f(p_pos->operator[](0), p_pos->operator[](1), p_pos->operator[](2)));
    
    // parent rotation matrix
    SbMatrix rotMatrix;
    rotMatrix.makeIdentity();
    if( p_rot->operator[](0) != 0.0 || p_rot->operator[](1) != 0.0 || p_rot->operator[](2) != 0.0)
      rotMatrix.setRotate(SbRotation(SbVec3f(p_rot->operator[](0), p_rot->operator[](1), p_rot->operator[](2)), -p_rot->operator[](3)));

    // parent scale matrix
    SbMatrix sclMatrix;
    sclMatrix.setScale(SbVec3f(p_scl->operator[](0), p_scl->operator[](1), p_scl->operator[](2)));

    // perform child translation transformations in right order
    SbVec3f c_pos;    
    sclMatrix.multVecMatrix(SbVec3f(kfmesh->pos[0].x, kfmesh->pos[0].y, kfmesh->pos[0].z), c_pos);
    rotMatrix.multVecMatrix(c_pos, c_pos);
    posMatrix.multVecMatrix(c_pos, *p_pos);
    



    // child rotation matrix
    SbMatrix aux_matrix;
    aux_matrix.makeIdentity();
    if( kfmesh->rot[0].x != 0.0 || kfmesh->rot[0].y != 0.0 || kfmesh->rot[0].z != 0.0 )
      aux_matrix.setRotate(SbRotation(SbVec3f(kfmesh->rot[0].x, kfmesh->rot[0].y, kfmesh->rot[0].z), kfmesh->rot[0].angle));
    
    // parent rotation matrix (not negative angle)
    rotMatrix.makeIdentity();
    if( p_rot->operator[](0) != 0.0 || p_rot->operator[](1) != 0.0 || p_rot->operator[](2) != 0.0)
      rotMatrix.setRotate(SbRotation(SbVec3f(p_rot->operator[](0), p_rot->operator[](1), p_rot->operator[](2)), p_rot->operator[](3)));
    // rotation composition
    rotMatrix.multRight(aux_matrix);


    // obtain rotation from matrix
    SbVec3f pos;
    SbVec4f rot;
    SbVec3f scl;
    SbRotation r;
    SbRotation so;
    SbVec3f rot2;
    float angle;
   
    rotMatrix.getTransform(pos, r, scl, so);
    r.getValue(rot2, angle);
    p_rot->setValue(rot2.operator[](0), rot2.operator[](1), rot2.operator[](2), angle);

    
    // child rotation matrix
    rotMatrix.makeIdentity();    
    if( kfmesh->rot[0].x != 0.0 || kfmesh->rot[0].y != 0.0 || kfmesh->rot[0].z != 0.0 )
      rotMatrix.setRotate(SbRotation(SbVec3f(kfmesh->rot[0].x, kfmesh->rot[0].y, kfmesh->rot[0].z), -kfmesh->rot[0].angle));
    // composition child rotation and parent's scale
    rotMatrix.multRight(sclMatrix);
    rotMatrix.getTransform(pos,r,*p_scl,so);
    // composition child scale and transformed scale of parent
    p_scl->setValue(p_scl->operator[](0)*kfmesh->scale[0].x, p_scl->operator[](1)*kfmesh->scale[0].y, p_scl->operator[](2)*kfmesh->scale[0].z);

  }
  else
  {
    // parent translation, rotation and scale
    p_pos->setValue(kfmesh->pos[0].x, kfmesh->pos[0].y, kfmesh->pos[0].z);
    p_rot->setValue(kfmesh->rot[0].x, kfmesh->rot[0].y, kfmesh->rot[0].z, kfmesh->rot[0].angle);
    p_scl->setValue(kfmesh->scale[0].x, kfmesh->scale[0].y, kfmesh->scale[0].z);
  }
}

/*
 * Add model transformation into scene graph. This part solving instances, hierarchy, bugs and so on
 */
void keyFrameClass::performKFTransformations(int *clockshape) {

  if( kfmesh != NULL )
  {
    SbVec3f pos(0,0,0);
    SbVec4f rot(0,0,0,0);
    SbVec3f scl(0,0,0);

    // find the root for actual model
    findTopParent(clockshape, &pos, &rot, &scl, kfmesh);
    
    // instance translation
    SoTransform *transform4 = new SoTransform();
      transform4->translation.setValue(SbVec3f(pos.operator[](0) - mesh->locmatrix[9], pos.operator[](1) - mesh->locmatrix[10], pos.operator[](2) - mesh->locmatrix[11]));
	    transform4->center.setValue(mesh->locmatrix[9], mesh->locmatrix[10], mesh->locmatrix[11]);
    meshSet->addChild(transform4);
    
    // rotation
    if( rot.operator[](0) != 0.0 || rot.operator[](1) != 0.0 || rot.operator[](2) != 0.0 )
    {	  
	    SoTransform *transform1 = new SoTransform();
        transform1->rotation.setValue(SbVec3f(rot.operator[](0), rot.operator[](1), rot.operator[](2)), -rot.operator[](3));
        transform1->center.setValue(mesh->locmatrix[9], mesh->locmatrix[10], mesh->locmatrix[11]);
      meshSet->addChild(transform1);
    }

    // scale - eliminate bug in 3ds file format
    if( (scl.operator[](0) * scl.operator[](1) * scl.operator[](2)) < 0.0 )
    {	  
	    *clockshape = 1;
	    SoTransform *transform3 = new SoTransform();
        transform3->scaleFactor.setValue(SbVec3f(-1,1,1));
	      transform3->center.setValue(mesh->locmatrix[9], mesh->locmatrix[10], mesh->locmatrix[11]);  
      meshSet->addChild(transform3);
    }

    // scale
	  SoTransform *transform2 = new SoTransform();
      transform2->scaleFactor.setValue(scl.operator[](0), scl.operator[](1), scl.operator[](2));
	    transform2->center.setValue(mesh->locmatrix[9], mesh->locmatrix[10], mesh->locmatrix[11]);
    meshSet->addChild(transform2); 


    // inverse matrix related for pivot point. Add negative translation to pivot too
    SbMatrix matrix(
	    mesh->locmatrix[0], mesh->locmatrix[1], mesh->locmatrix[2], 1,
	    mesh->locmatrix[3], mesh->locmatrix[4], mesh->locmatrix[5], 1,
	    mesh->locmatrix[6], mesh->locmatrix[7], mesh->locmatrix[8], 1,
	    0, 0, 0, 1
	  );
	  matrix = matrix.inverse();
	  SoTransform *transform = new SoTransform();
      transform->setMatrix(matrix);
      transform->translation.setValue(-kfmesh->pivot.x, -kfmesh->pivot.y, -kfmesh->pivot.z); // jeste pivot
      transform->center.setValue(mesh->locmatrix[9], mesh->locmatrix[10], mesh->locmatrix[11]);
    meshSet->addChild(transform);
  }
}

/*
 * Constructor: filling some needed attributes (varibles) for this class
 */
keyFrameClass::keyFrameClass(sceneClass *sceneCls, int index) {

  // mapping needed attributes
  meshSet = sceneCls->getMeshSet();
  mesh = sceneCls->getMesh();
    

  kfmesh = sceneCls->getKfmesh();
  kfmodel = sceneCls->getModel();
}

keyFrameClass::~keyFrameClass() {
}
