/*****************************************************************************\
 *
 * materialClass.cpp
 *
 * Updated: 14.12.2005
 * 
 * materialClass  - creates base material and material properties.
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "materialClass.h"
/*
 * Add shapeHints node to scene graph.
 * Set TwoSideLightning a BackFaceCulling
 * There was used :
 * if there is 2sided, then we can se both side of face, else we can se only one side of face
 * If there is no defined material for this object, then use default material with one sided lightning.
 */
void materialClass::addMaterialAttr(int attr, int clockshape)
{
  SoShapeHints *twoSided = new SoShapeHints();

  if( param.twoSided )
  {
    twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
    twoSided->vertexOrdering = clockshape == 0 ? SoShapeHints::COUNTERCLOCKWISE : SoShapeHints::CLOCKWISE;
    twoSided->creaseAngle = (float) 0.0;

    materialSet->addChild(twoSided);
  }
  else
    switch( attr )
    {
      // twosidelinghtning on and backface culling off
      case MY_MAT_TWO_SIDED:
        if( material->twosided )
        {
          twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
          twoSided->vertexOrdering = clockshape == 0 ? SoShapeHints::COUNTERCLOCKWISE : SoShapeHints::CLOCKWISE;
          twoSided->creaseAngle = (float) 0.0;

          materialSet->addChild(twoSided);
        }
        else // twosidelinghtning off and backface culling on
        {
          twoSided->shapeType = SoShapeHints::SOLID;
          twoSided->vertexOrdering = clockshape == 0 ? SoShapeHints::COUNTERCLOCKWISE : SoShapeHints::CLOCKWISE;
          twoSided->creaseAngle = (float) 0.0;

          materialSet->addChild(twoSided);
        }
      break;

      // 3DS default: twosidelinghtning off and backface culling on
      case MY_MAT_DEFAULT:
        twoSided->shapeType = SoShapeHints::SOLID;
        twoSided->vertexOrdering = clockshape == 0 ? SoShapeHints::COUNTERCLOCKWISE : SoShapeHints::CLOCKWISE;
        twoSided->creaseAngle = (float) 0.0;

        materialSet->addChild(twoSided);
      break;

    default:break;
  }
}

/*
 * This function set base material attributes to actual object.
 * If tehere is no defined material for this object, then use my own default material.
 */
void materialClass::addMaterial(int clockshape)
{
  SoMaterial *meshMat = new SoMaterial();

  // if tehre is some material defined, load it else use default material
  if( mesh->nmats != 0)
  {

    // material attributes supported by Open Inventor
    meshMat->ambientColor.setValue(material->ambient.r, material->ambient.g, material->ambient.b);
    meshMat->diffuseColor.setValue(material->diffuse.r, material->diffuse.g, material->diffuse.b);
    meshMat->specularColor.setValue(material->specular.r, material->specular.g, material->specular.b);
    meshMat->shininess.setValue(material->shininess);

    meshMat->emissiveColor.setValue(material->diffuse.r * material->selfillumpct,
                                    material->diffuse.g * material->selfillumpct, 
                                    material->diffuse.b * material->selfillumpct);

    meshMat->transparency.setValue(material->transparency < 0.0 ? -material->transparency : material->transparency);

    // try to add material without default values
    addMaterialAttr(MY_MAT_TWO_SIDED, clockshape);

    SoDrawStyle *style = new SoDrawStyle;

    // object shading method
    switch(material->shading)
    {
      // style wireframe
      case(0):

        //printf("bounds[0]=%f\n",minLW);
        //printf("bounds[1]=%f\n",maxLW);

        style->style = SoDrawStyle::LINES;

        // set line width in relation with system / 3D Studio
        // system has minLW..maxLW and 3DS has 0..100
        if( material->wiresize == 0.0 )
          style->style = SoDrawStyle::INVISIBLE;
        else
        {
          float lw = 0;
          if( maxLW != 0 )
          {
            lw = 100 / maxLW;
            lw = material->wiresize / (float) lw;
            //printf("lw=%f\n",lw);
            if( lw < minLW )
              style->lineWidth = minLW;
            else
             style->lineWidth = lw;
          }
          //else default
        }

      break;

      // default draw style
      default:
        style->style = SoDrawStyle::FILLED;
      break;
    }

    materialSet->addChild(style);
  }
  else
  {
    // default material
    meshMat->ambientColor.setValue(1.0,0.0,0.0);
    meshMat->diffuseColor.setValue(1.0,0.0,0.0);
    meshMat->specularColor.setValue(1.0,0.0,0.0);
    meshMat->shininess.setValue( (float) 0.1);
    meshMat->transparency.setValue(0.0);

    addMaterialAttr(MY_MAT_DEFAULT, clockshape);
  }

  materialSet->addChild(meshMat);
}

/*
 * Constructor: fill some atributes for materialClass use
 */
materialClass::materialClass(sceneClass *sceneCls) {

  // mapping needed atributes
  mesh = sceneCls->getMesh();
  material = sceneCls->getMaterial();
  materialSet = sceneCls->getMaterialSet();
  
  minLW = sceneCls->minLW;
  maxLW = sceneCls->maxLW;
  
  param = sceneCls->getParam();
  
}

materialClass::~materialClass() {
}


