/*****************************************************************************\
 *
 * normalClass.h
 *
 * Updated: 14.12.2005
 * 
 * normalClass  - calss for computing face and vertex normals
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef _normalClass_h_
#define _normalClass_h_

#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>

#include "lib3ds.h"
#include "smoothGroupsClass.h"
#include "geometryClass.h"

/*
 * Class for computing face and vertices normal
 */
class normalClass {
  
  meshLib3               *mesh;
  smoothGroupsClass      *smGroup;
  SbList<tSmoothGroups>  *augumentVertices;

  // compute vector from two input points
  void vectorOffset(pointLib3 *pIn, pointLib3 *pOffset, pointLib3 *pOut);
  // compute normal for two vectors
  void vectorGetNormal(pointLib3 *a, pointLib3 *b, pointLib3 *pOut);

public:
  
  // compute face normal from three points
  void computeFaceNormal(pointLib3 *p1, pointLib3 *p2, pointLib3 *p3, pointLib3 *pOut);
  // compute vertex normal based on vertices index and matching smoothing group
  void computeVertexNormal(int vertexIndex, int smgroupIndex, pointLib3 *pOut);
  
  normalClass();
  normalClass(geometryClass *);
  ~normalClass() {};
};

#endif
