#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "meshDatabase.h"

/*
 * Add mesh to database and add this mesh's name to meshList
 * Init mesh default's values too
 */
void meshDatabaseLib3::addMesh(char *name) {
  int i;

  // alocate memory for new mesh 
  meshList.count += 1;
  meshList.list = (nameListLib3 *) realloc(meshList.list, sizeof(nameListLib3) * meshList.count);
  mesh = (meshLib3 *) realloc(mesh, sizeof(meshLib3) * meshList.count);

  // add mesh to meshList
  strcpy(meshList.list[meshList.count - 1].name, name);
  strcpy(mesh[meshList.count - 1].name, name);

  // set mesh default values;
  mesh[meshList.count - 1].nmats = 0;
  mesh[meshList.count - 1].nvertices = 0;
  mesh[meshList.count - 1].nvflags = 0;
  mesh[meshList.count - 1].nfaces = 0;
  mesh[meshList.count - 1].ntextverts = 0;
     
  mesh[meshList.count - 1].vertexarray = NULL;
  mesh[meshList.count - 1].vflagarray = NULL;
  mesh[meshList.count - 1].facearray = NULL;
  mesh[meshList.count - 1].matarray = NULL;
  mesh[meshList.count - 1].smootharray = NULL;
  mesh[meshList.count - 1].textarray = NULL;

  for( i = 0; i < LOCAL_MATRIX_SIZE; i++ ) mesh[meshList.count-1].locmatrix[i] = 0.0;
  
}

/*
 * Add vertex array to actual mesh and init nvertices attribute
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addPointArray() {
  int i;
  unsigned short int readShort;

  fread(&readShort, sizeof(unsigned short int), 1, inputFile);
  mesh[meshList.count - 1].nvertices = readShort;
  mesh[meshList.count - 1].vertexarray = (pointLib3 *) malloc( sizeof(pointLib3) * readShort);

  for( i = 0; i < readShort; i++)
  {
	  fread(&mesh[meshList.count - 1].vertexarray[i].x, sizeof(float), 1, inputFile);
    fread(&mesh[meshList.count - 1].vertexarray[i].y, sizeof(float), 1, inputFile);
	  fread(&mesh[meshList.count - 1].vertexarray[i].z, sizeof(float), 1, inputFile);
  }
}


/*
 * Add point flag array to actual mesh and init nvflags attribute
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addPointFlagArray() {
  int i;
  unsigned short int readShort;

	fread(&readShort, sizeof(unsigned short int), 1, inputFile);
  mesh[meshList.count - 1].nvflags = readShort;
  mesh[meshList.count - 1].vflagarray = (unsigned short int *) malloc( sizeof(unsigned short int) * readShort);
                
  for( i = 0; i < readShort; i++)
   fread(&mesh[meshList.count - 1].vflagarray[i], sizeof(unsigned short int), 1, inputFile);
}

/*
 * Add face array to actual mesh and init nfaces attribute
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addFaceArray() {
  int i;
  unsigned short int readShort;

  fread(&readShort, sizeof(unsigned short int), 1, inputFile);
  mesh[meshList.count - 1].nfaces = readShort;
  mesh[meshList.count - 1].facearray = (faceLib3 *) malloc( sizeof(faceLib3) * readShort);
  
  for( i = 0; i < readShort; i++)
  {
	  fread(&mesh[meshList.count - 1].facearray[i].v1, sizeof(unsigned short int), 1, inputFile);
    fread(&mesh[meshList.count - 1].facearray[i].v2, sizeof(unsigned short int), 1, inputFile);
	  fread(&mesh[meshList.count - 1].facearray[i].v3, sizeof(unsigned short int), 1, inputFile);
    fread(&mesh[meshList.count - 1].facearray[i].flag, sizeof(unsigned short int), 1, inputFile);
  }
}

/*
 * Add mesh mat group to actual mesh and increment nmats for this mesh
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addMeshMatGroup() {
  int i = 0;

  char name[MATERIAL_NAME_LIB_3DS_LEN];

  mesh[meshList.count - 1].nmats += 1;
  mesh[meshList.count - 1].matarray = (matLib3 *) realloc(mesh[meshList.count - 1].matarray, sizeof(matLib3) * mesh[meshList.count - 1].nmats);

  do fread (&name[i], 1, 1, inputFile);
  while(name[i++] != '\0' && i < MATERIAL_NAME_LIB_3DS_LEN);

  strcpy(mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].name, name);
               
  //count of actual material's faces
  fread(&mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].nfaces, sizeof(unsigned short int), 1, inputFile);
  

  //actual material's faces number
  mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].faceindex = (unsigned short int *) malloc( sizeof(unsigned short int) * mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].nfaces);
  for( i = 0; i < mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].nfaces; i++) 
	  fread(&mesh[meshList.count - 1].matarray[mesh[meshList.count - 1].nmats - 1].faceindex[i], sizeof(unsigned short int), 1, inputFile);
}


/*
 * Add smooth array for this mesh (whitch define smoothing groups)
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addSmoothArray() {
  int i;
 
  mesh[meshList.count - 1].smootharray = (unsigned long int *) malloc(sizeof(unsigned long int) * mesh[meshList.count - 1].nfaces);

  for( i = 0; i < mesh[meshList.count - 1].nfaces; i++) 
	  fread(&mesh[meshList.count - 1].smootharray[i], sizeof(unsigned long int), 1, inputFile);
}


/*
 * Add texture vertices array for this mesh
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addTextVertArray() {
  int i;

  fread(&mesh[meshList.count - 1].ntextverts, sizeof(unsigned short int), 1, inputFile);
          
  mesh[meshList.count - 1].textarray = (textLib3 *) malloc(sizeof(textLib3) * mesh[meshList.count - 1].ntextverts);

  for( i = 0; i < mesh[meshList.count - 1].ntextverts; i++) {
	  fread(&mesh[meshList.count - 1].textarray[i].u, sizeof(float), 1, inputFile);
    fread(&mesh[meshList.count - 1].textarray[i].v, sizeof(float), 1, inputFile);
	}
}

/*
 * Add mesh's local transformation matrix
 * This function read data from inputFile.
 */
void meshDatabaseLib3::addMeshMatrix() {
  fread(mesh[meshList.count - 1].locmatrix, sizeof(float), LOCAL_MATRIX_SIZE, inputFile);
}

/*
 * Get pointer to meshList from database
 */
meshListLib3 *meshDatabaseLib3::dbGetMeshList() {
  return(&meshList);
}

/*
 * Get pointer to mesh from database
 */
meshLib3 *meshDatabaseLib3::dbGetMesh(int i) {
  return(&mesh[i]);
}

/*
 * mesh database constructor
 */
meshDatabaseLib3::meshDatabaseLib3(FILE *inpFile) {
  
  meshList.list = NULL;
  meshList.count = 0;
  
  mesh = NULL;
  
  inputFile = inpFile;
};

/*
 * mesh database destructor
 */
meshDatabaseLib3::~meshDatabaseLib3() {
  int i, j;
  
  if( mesh != NULL ) 
  {
    for( i = 0; i < meshList.count; i++)
    {
      if( mesh[i].vertexarray != NULL ) free(mesh[i].vertexarray);
      if( mesh[i].vflagarray != NULL ) free(mesh[i].vflagarray);
      if( mesh[i].facearray != NULL ) free(mesh[i].facearray);
      if( mesh[i].matarray != NULL ) {
        for( j = 0; j < mesh[i].nmats; j++ )
          free(mesh[i].matarray[j].faceindex);
        free(mesh[i].matarray);
      }
      if( mesh[i].smootharray != NULL ) free(mesh[i].smootharray);
      if( mesh[i].textarray != NULL ) free(mesh[i].textarray);
    }
    free(mesh);
  }
  if( meshList.list != NULL ) free(meshList.list);
};
