/*****************************************************************************\
 *
 * geometryClass.cpp
 *
 * Updated: 14.12.2005
 * 
 * geometryClass  - creates geometry part of scene.
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/
#include "normalClass.h"
#include "geometryClass.h"

/*
 * This function add information about this object's triangles
 * Parameters are faces, smg separator for distinguiise between smoothing groups
 * and the last parameter is coutn of vertex in each smooth group
 */
void geometryClass::addIndexedFaceSet(int *faces, SoSeparator *smg, int csmgr)
{
  SoIndexedFaceSet *meshFaceSet = new SoIndexedFaceSet();
  meshFaceSet->coordIndex.setNum(csmgr*4);
  meshFaceSet->coordIndex.setValues(0, csmgr*4, faces);
  // add triangles (faces) to smg separator
  smg->addChild(meshFaceSet);
  // add smg separator to materialSet separator
  materialSet->addChild(smg);
}

/*
 * Add vertices coordinate into meshSet separator
 */
void geometryClass::addCoordinate()
{
  SoCoordinate3 *vertices = new SoCoordinate3();

  vertices->point.setNum(mesh->nvertices);
  vertices->point.setValues(0, mesh->nvertices, (SbVec3f *) mesh->vertexarray);
  meshSet->addChild(vertices);
}

/*
 * Refresh pointer to mesh's part
 */
void geometryClass::initBit(SoSeparator *matSet) {
  materialSet = matSet;
}

/*
 * Function for computing and calling next other function for create complete 
 * geometry for mesh's part.
 */
void geometryClass::createGeometry(int mat_id, int faceted)
{
  // add all vertex points to scene graph
  addCoordinate();

  // auxiliary array for storing count of faces be,ongs to semarately smooth groups
  // Index 0 means, that face belong to nothing smoothing groups
  unsigned long int csmgr[MAX_SMOOTH_GROUPS];

  // array for store faces divided by smoothing groups info
  int **faces;
 
  // compute size of allocated array. If object has some defined material,
  // then alloc needed bytes to store this material's faces else  
  // allocace space for all faces belongs to this mesh
  unsigned int n_faces = mesh->nmats != 0 ? mesh->matarray[mat_id].nfaces : mesh->nfaces;

  // allocate memory for 2d faces array [n_faces * 4][MAX_SMOOTH_GROUPS] 
  unsigned long int k;
  faces = new int *[MAX_SMOOTH_GROUPS];
  for(k = 0; k < MAX_SMOOTH_GROUPS; k++)
    faces[k] = new int [n_faces * 4];
  
  // reset faces counter
  int i;
  for(i = 0; i < MAX_SMOOTH_GROUPS; i++) 
    csmgr[i] = 0;

  for(k = 0; k < n_faces; k++)
  {     
      // uncover vertex index for processing
      int aux_idx = mesh->nmats != 0 ? mesh->matarray[mat_id].faceindex[k] : k;

      // uncover, what sm group entered faces belongs
      SbIntList *smgr = smGroup->getSmoothingGroups(aux_idx);
      // for all sm groups, in which this faces belong, create items in corresponding faces array
      // and acualize csmgr counters

      int m;
      int length = smgr->getLength();
      for(m = 0; m < length; m++)
      {
        int ismgr;
        // application faceted parameter of material
        ismgr = (int) smgr->get(m); // read the sm group identificator
        // store defined face's vertex to face's array
        // separate by -1 for solve Open Inventoru scene graph creation
        // for indexedFaceSet
        faces[ismgr][csmgr[ismgr] * 4 + 0] = mesh->facearray[aux_idx].v1;
        faces[ismgr][csmgr[ismgr] * 4 + 1] = mesh->facearray[aux_idx].v2;
        faces[ismgr][csmgr[ismgr] * 4 + 2] = mesh->facearray[aux_idx].v3;
        faces[ismgr][csmgr[ismgr] * 4 + 3] = -1;      
        csmgr[ismgr] += 1;
      }
  }
     
  // auxiliary variable
  pointLib3 auxPoint;

  // if object contain some faces withou defined sm groups, then process it (flat shading)

  // create object for compute normals
  normalClass *normalCls = new normalClass(this);

  if( csmgr[0] != 0 )
  {
    // allocate memory for computed normals
    pointLib3 *anormals = new pointLib3 [csmgr[0]*3];
    // for all faces compute one normal
    for(k = 0; k < csmgr[0]; k++)
    {
      normalCls->computeFaceNormal(&mesh->vertexarray[faces[0][k*4 + 0]],
                        &mesh->vertexarray[faces[0][k*4 + 1]],
                        &mesh->vertexarray[faces[0][k*4 + 2]],
                        &auxPoint);
      // store computed normal into output array
      anormals[k] = auxPoint;
    }

    // create separator of actual sm group (including no smg)
    SoSeparator *smg = new SoSeparator;
    // add and set binding container
    SoNormalBinding *nbind = new SoNormalBinding;
    nbind->value.setValue(SoNormalBinding::PER_FACE);
    smg->addChild(nbind);

    // create container for normals
    SoNormal *normals = new SoNormal;
    // add normals to container
    normals->vector.setNum(csmgr[0]);
    normals->vector.setValues(0, csmgr[0], (SbVec3f *) anormals);
    // add noramls container into smg separator
    smg->addChild(normals);

    // free memory for noramls
    delete [] anormals;

    // add face definition (V1,V2,V3,-1) to smg separator
    addIndexedFaceSet(faces[0], smg, csmgr[0]);
  }    


  // for all faces, which belong to some sm group
  for(i = 1; i < MAX_SMOOTH_GROUPS; i++)
  {     
    // if there is some faces on this sm group, then process it
    if( csmgr[i] != 0)
    {
       // enabled parameter faceted in command line or directly from 3D Studio
      if( !(faceted == 1 || param.faceted) )
      {
        // allocate memory for computed normals
        pointLib3 *vertNormals = new pointLib3 [csmgr[i]*3];

        // for all faces in actual sm group compute norals per vertex
        for(k = 0; k < csmgr[i]; k++)
        {       
          // compute vertex normal, parameter is vertex's index, actual sm group
          // and we expected output in auxPoint format
          normalCls->computeVertexNormal(faces[i][k*4 + 0], i, &auxPoint);
          vertNormals[k*3 + 0] = auxPoint;

          // equals code above, but for second vertex
          normalCls->computeVertexNormal(faces[i][k*4 + 1], i, &auxPoint);
          vertNormals[k*3 + 1] = auxPoint;

          // equals code above, but for thirth vertex
          normalCls->computeVertexNormal(faces[i][k*4 + 2], i, &auxPoint);
          vertNormals[k*3 + 2] = auxPoint;
        }

        // create separator for actual sm group (1 - MAX_SMOOTH_GROUPS)
        SoSeparator *smg = new SoSeparator;
        // set and add binding for normal/vertex to smg separator
        SoNormalBinding *nbind = new SoNormalBinding;
        nbind->value.setValue(SoNormalBinding::PER_VERTEX);
        smg->addChild(nbind);

        // create container for normals
        SoNormal *normals = new SoNormal;
        // add normals into these container
        normals->vector.setNum(csmgr[i]*3);
        normals->vector.setValues(0, csmgr[i]*3, (SbVec3f *) vertNormals);
        // add noramls sontainer in to smg separator
        smg->addChild(normals);

        // release memory alloceted for normals
        delete [] vertNormals;

        // add face definition (V1,V2,V3,-1) into smg container 
        addIndexedFaceSet(faces[i], smg, csmgr[i]);
      }
      else // faceted
      {
        // alloc memory for computed noramls
        pointLib3 *anormals = new pointLib3 [csmgr[i]*3];

        // compute normal for all triangles in face array
        for(k = 0; k < csmgr[i]; k++)
        {
          normalCls->computeFaceNormal(&mesh->vertexarray[faces[i][k*4 + 0]],
                          &mesh->vertexarray[faces[i][k*4 + 1]],
                          &mesh->vertexarray[faces[i][k*4 + 2]],
                          &auxPoint);
          // store computed normal into output array
          anormals[k] = auxPoint;
        }

        // vytvor kontejner pro prislusnou smg skupinu (i zadna)
        SoSeparator *smg = new SoSeparator;
        // nastav a pridej do kontejneru binding na normala/trojuhelnik
        SoNormalBinding *nbind = new SoNormalBinding;
        nbind->value.setValue(SoNormalBinding::PER_FACE);
        smg->addChild(nbind);

        // create container for normals
        SoNormal *normals = new SoNormal;
        normals->vector.setNum(csmgr[i]);
        normals->vector.setValues(0, csmgr[i], (SbVec3f *) anormals);
        // add normal's container to smg separator
        smg->addChild(normals);

        // free memory for normals
        delete [] anormals;

        // add face definition (V1,V2,V3,-1) into smg container 
        addIndexedFaceSet(faces[i], smg, csmgr[i]);
      }
    }    
  }  

  // release all alocated memory for triangles (faces)
  for(k = 0; k < MAX_SMOOTH_GROUPS; k++)
    delete [] faces[k];
  delete [] faces;
}

/*
 * Function return a mesh structure stored in sceneClass
 */
meshLib3 *geometryClass::getMesh() {
  return mesh;
};

smoothGroupsClass *geometryClass::getSmGroup() {
  return(smGroup); 
};

geometryClass::geometryClass(sceneClass *sceneCls) {
  
  mesh = sceneCls->getMesh();
  meshSet = sceneCls->getMeshSet();
  materialSet = sceneCls->getMaterialSet();

  // create object for working with smoothing groups
  smGroup = new smoothGroupsClass(mesh);
  smGroup->performAugumentation();
}

geometryClass::~geometryClass() {
  
  // delete object for working with smoothing groups
  if( smGroup != NULL ) {
    delete smGroup;
    smGroup = NULL;
  }
}