/*****************************************************************************\
 *
 * textureClass.cpp
 *
 * Updated: 14.12.2005
 *
 * testureClass  - creates texture and texture properties.
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "textureClass.h"

/*
 *  This function settings actual texture's parameters
 */
void textureClass::addTextureAttr()
{  
  if( material->texture.map.name != NULL )
  {
    SoTexture2Transform *textTransf = new SoTexture2Transform();
    textTransf->translation.setValue(material->texture.map.uoffset, material->texture.map.voffset);
    textTransf->scaleFactor.setValue(material->texture.map.uscale, material->texture.map.vscale);
    materialSet->addChild(textTransf);
  }
  
  if( material->bump.map.name != NULL)
  {
    SoBumpMapTransform *textTransfB = new SoBumpMapTransform();
    textTransfB->translation.setValue(material->bump.map.uoffset, material->bump.map.voffset);
    textTransfB->scaleFactor.setValue(material->bump.map.uscale, material->bump.map.vscale);
    materialSet->addChild(textTransfB); 
  }  
}

/*
 * Add texture and texture binding to scene graph
 * texturovaci binding. 
 */
void textureClass::addTexture()
{
  
  addTextureAttr();

  std::string pokus = material->texture.map.name;
  if( pokus.length() != 0)  
  {
    FILE *exist = NULL;
    if( (exist = fopen(material->texture.map.name,"rb")) != NULL)
    {
      fclose(exist);
      SoTexture2 *texture = new SoTexture2();
      texture->filename.setValue(material->texture.map.name);
      texture->model = material->texture.map.source == 0 ? SoTexture2::DECAL : SoTexture2::MODULATE; 
      materialSet->addChild(texture);
    }
    else
      std::cerr << "Cannot find texture file \"" << material->texture.map.name << "\".\n";
  }

/*
  if( material->opacity.map.name != NULL )
  {
    SoTextureUnit *tu = new SoTextureUnit;
    tu->unit = 1; // set texture unit to 1
    tu->mappingMethod = SoTextureUnit::IMAGE_MAPPING;
    whatMesh->addChild(tu);

    SoTexture2 *texture = new SoTexture2();
    texture->filename.setValue(material->opacity.map.name);
    whatMesh->addChild(texture);

    SoTextureCombine *tc = new SoTextureCombine;
    tc->alphaSource = SoTextureCombine::TEXTURE;
    tc->alphaOperand = SoTextureCombine::ONE_MINUS_SRC_ALPHA;
    tc->alphaOperation = SoTextureCombine::DOT3_RGBA;
    whatMesh->addChild(tc);

    tu = new SoTextureUnit;
    tu->unit = 0; // set texture unit to 1
    tu->mappingMethod = SoTextureUnit::IMAGE_MAPPING;
    whatMesh->addChild(tu);
  }
  */

  pokus = material->bump.map.name;
  if( pokus.length() != 0)
  {
    FILE *exist = NULL;
    if( (exist = fopen(material->bump.map.name,"rb")) != NULL)
    {
      fclose(exist);
      SoBumpMap *textureB = new SoBumpMap();
      textureB->filename.setValue(material->bump.map.name);
      materialSet->addChild(textureB);
    }
    else
      std::cerr << "Cannot find texture file \"" << material->bump.map.name << "\".\n";
  }

}

/*
 *  Add texture coordinato to scene graph
 */
void textureClass::addTextureCoords()
{	
  if( mesh->textarray != NULL)
  {
    SoTextureCoordinate2 *textCoords = new SoTextureCoordinate2();

    textCoords->point.setNum(mesh->ntextverts);
    textCoords->point.setValues(0, mesh->ntextverts, (SbVec2f *) mesh->textarray);
    meshSet->addChild(textCoords);

    SoTextureCoordinateBinding *tBind = new SoTextureCoordinateBinding();
    tBind->value.setValue(SoTextureCoordinateBinding::PER_VERTEX_INDEXED);
    meshSet->addChild(tBind);  
  }  
}

/*
 * Constructor: set some attributes for using in material
 */
textureClass::textureClass(sceneClass *sceneCls) {

  // mapovani potrebnych atributu
  meshSet = sceneCls->getMeshSet();
  mesh = sceneCls->getMesh();
  materialSet = sceneCls->getMaterialSet();
  material = sceneCls->getMaterial();

  addTextureCoords();
}

/*
 * Destructor is empty
 */
textureClass::~textureClass() {
}
