#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "kframeDatabase.h"

/*
 * This function create new key frame node
 */
void kframeDatabaseLib3::addKFandNode(int readNodeId) {

  // couner of nodes in 3ds file
  kframeList.count += 1;
  // alocate memory for each node
  kframe = (kfmeshLib3 *) realloc(kframe, sizeof(kfmeshLib3) * kframeList.count);

  kframe[kframeList.count - 1].pos = NULL;
  kframe[kframeList.count - 1].pkeys = NULL;
  kframe[kframeList.count - 1].rot = NULL;
  kframe[kframeList.count - 1].rkeys = NULL;
  kframe[kframeList.count - 1].scale = NULL;
  kframe[kframeList.count - 1].skeys = NULL;
  kframe[kframeList.count - 1].instance[0] = '\0';
  kframe[kframeList.count - 1].parent[0] = '\0';
  kframe[kframeList.count - 1].boundmin.x = 0;
  kframe[kframeList.count - 1].boundmin.y = 0;
  kframe[kframeList.count - 1].boundmin.z = 0;
  kframe[kframeList.count - 1].boundmax.x = 0;
  kframe[kframeList.count - 1].boundmax.y = 0;
  kframe[kframeList.count - 1].boundmax.z = 0;
  kframe[kframeList.count - 1].flags1 = 0;
  kframe[kframeList.count - 1].flags2 = 0;
  kframe[kframeList.count - 1].name[0] = '\0';
  kframe[kframeList.count - 1].nodeid = 65535;
  kframe[kframeList.count - 1].npflag = 0;
  kframe[kframeList.count - 1].npkeys = 0;
  kframe[kframeList.count - 1].nrflag = 0;
  kframe[kframeList.count - 1].nrkeys = 0;
  kframe[kframeList.count - 1].nsflag = 0;
  kframe[kframeList.count - 1].nskeys = 0;
  kframe[kframeList.count - 1].pivot.x = 0;
  kframe[kframeList.count - 1].pivot.y = 0;
  kframe[kframeList.count - 1].pivot.z = 0;
  kframe[kframeList.count - 1].sparent = 65535;

  // read node id from 3ds file
  if( readNodeId ) 
    fread(&kframe[kframeList.count - 1].nodeid, sizeof(unsigned short int), 1, inputFile);
  else
    kframe[kframeList.count - 1].nodeid = kframeList.count - 1;

  nodeIdFound = 1;
};

/*
 * This function read header for actual key frame node. There is information about
 * node name, some flags and this node's parent.
 */
void kframeDatabaseLib3::addHeader() {
  int i=0;
  char name[KFRAME_NAME_LIB_3DS_LEN];

  if( !nodeIdFound )addKFandNode(0);

  // read node name from 3ds file
  do fread(&name[i], sizeof(unsigned char), 1, inputFile);
  while(name[i++] != '\0' && i < KFRAME_NAME_LIB_3DS_LEN);  
  
  strcpy(kframe[kframeList.count - 1].name, name);

  // read flags1
  fread(&kframe[kframeList.count - 1].flags1, sizeof(unsigned short int), 1, inputFile);

  // read flags2
  fread(&kframe[kframeList.count - 1].flags2, sizeof(unsigned short int), 1, inputFile);

  // read node's parent from 3ds file as index
  fread(&kframe[kframeList.count - 1].sparent, sizeof(unsigned short int), 1, inputFile);

  // search for the node's parent's name using read index
  // node name + dot + instance name -> 22 chars
  kframe[kframeList.count - 1].parent[0] = '\0';
  if( kframe[kframeList.count - 1].sparent != 65535 )
  {
    for( i = 0; i < kframeList.count; i++)
      if( kframe[i].nodeid == kframe[kframeList.count - 1].sparent )
      {
         strcpy(kframe[kframeList.count - 1].parent, kframe[i].name);
         if( kframe[i].instance[0] != '\0') {
           strcat(kframe[kframeList.count - 1].parent, "." );
           strcat(kframe[kframeList.count - 1].parent, kframe[i].instance);
         }
         break;
      }
  }

  nodeIdFound = 0;
};

/*
 * This function get the pivot point for rotation and scale operations from 3ds file
 */
void kframeDatabaseLib3::addPivot() {
  fread(&kframe[kframeList.count - 1].pivot.x, sizeof(float), 1, inputFile);
  fread(&kframe[kframeList.count - 1].pivot.y, sizeof(float), 1, inputFile);
  fread(&kframe[kframeList.count - 1].pivot.z, sizeof(float), 1, inputFile);
};

/*
 * This function read an instance name for this keyframe node (respectively for
 * equaling mesh)
 */
void kframeDatabaseLib3::addInstanceName() {
  int i=0;
  char name[KFRAME_NAME_LIB_3DS_LEN];

  do fread(&name[i], sizeof(unsigned char), 1, inputFile);
  while(name[i++] != '\0' && i < KFRAME_NAME_LIB_3DS_LEN);  
 
  strcpy(kframe[kframeList.count - 1].instance, name);
};

/*
 * This function read a bounding box from 3ds file
 */
void kframeDatabaseLib3::addBoundingBox() {
    fread(&kframe[kframeList.count - 1].boundmin.x, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].boundmin.y, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].boundmin.z, sizeof(float), 1, inputFile);

		fread(&kframe[kframeList.count - 1].boundmax.x, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].boundmax.y, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].boundmax.z, sizeof(float), 1, inputFile);
}

/*
 * This function read a all postitions track tags for actual key frame
 */
void kframeDatabaseLib3::addPositionTT() {
  unsigned short int  readShort;
  unsigned long       readLong;
  unsigned int        i = 0;

  // first we have to read track header
  // uneccessary flags informations
  fread(&readShort, 2, 1, inputFile);

  // not currently used
  fread(&readLong, 4, 1, inputFile);

    // not currently used
  fread(&readLong, 4, 1, inputFile);

  // read number of keys in track
  fread(&kframe[kframeList.count - 1].npkeys, 4, 1, inputFile);
  	
  // alocate some memory for all the keys
  kframe[kframeList.count - 1].pos = (pointLib3 *) malloc(sizeof(pointLib3) * kframe[kframeList.count - 1].npkeys);
  kframe[kframeList.count - 1].pkeys = (keyHeaderLib3 *) malloc(sizeof(keyHeaderLib3) * kframe[kframeList.count - 1].npkeys);
 
  // read key header and immediately then read position and so on
  for(i = 0; i < kframe[kframeList.count - 1].npkeys; i++) {

    // init values;
    kframe[kframeList.count - 1].pkeys[i].tension = 0.0;
    kframe[kframeList.count - 1].pkeys[i].continuity = 0.0;
    kframe[kframeList.count - 1].pkeys[i].bias = 0.0;
    kframe[kframeList.count - 1].pkeys[i].easeto = 0.0;
    kframe[kframeList.count - 1].pkeys[i].easefrom = 0.0;


    // key header
    // time
    fread(&kframe[kframeList.count - 1].pkeys[i].time, sizeof(unsigned long int), 1, inputFile);
    // used parameters 0 - 20 bytes (0 - 5 x float)
    fread(&kframe[kframeList.count - 1].pkeys[i].rflags, sizeof(unsigned short int), 1, inputFile);

    // select key header used parameters base from rflags
    if( kframe[kframeList.count - 1].pkeys[i].rflags & 1 )   fread(&kframe[kframeList.count - 1].pkeys[i].tension, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].pkeys[i].rflags & 2 )   fread(&kframe[kframeList.count - 1].pkeys[i].continuity, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].pkeys[i].rflags & 4 )   fread(&kframe[kframeList.count - 1].pkeys[i].bias, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].pkeys[i].rflags & 8 )   fread(&kframe[kframeList.count - 1].pkeys[i].easeto, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].pkeys[i].rflags & 16 )  fread(&kframe[kframeList.count - 1].pkeys[i].easefrom, sizeof(float), 1, inputFile);

    kframe[kframeList.count - 1].pkeys[i].tension = kframe[kframeList.count - 1].pkeys[i].tension*25 + 25;
    kframe[kframeList.count - 1].pkeys[i].continuity = kframe[kframeList.count - 1].pkeys[i].continuity*25 + 25;
    kframe[kframeList.count - 1].pkeys[i].bias = kframe[kframeList.count - 1].pkeys[i].bias*25 + 25;
    kframe[kframeList.count - 1].pkeys[i].easeto = kframe[kframeList.count - 1].pkeys[i].easeto*50;
    kframe[kframeList.count - 1].pkeys[i].easefrom = kframe[kframeList.count - 1].pkeys[i].easefrom*50;

    // point (position)
    fread(&kframe[kframeList.count - 1].pos[i].x, sizeof(float), 1, inputFile);
    fread(&kframe[kframeList.count - 1].pos[i].y, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].pos[i].z, sizeof(float), 1, inputFile);
  }
};


/*
 * This function read a all rotations track tags for actual key frame
 */
void kframeDatabaseLib3::addRotationTT() {
  unsigned short int  readShort;
  unsigned long       readLong;
  unsigned int  i;

  // first we have to read track header
  
  // uneccessary informations
  fread(&readShort, sizeof(unsigned short int), 1, inputFile);

  // not currently used
  fread(&readLong, sizeof(unsigned long), 2, inputFile);
  
  // read number of keys in track
  fread(&kframe[kframeList.count - 1].nrkeys, sizeof(unsigned long), 1, inputFile);

  // alocate some memory for all the keys
  kframe[kframeList.count - 1].rot = (kfRotKeyLib3 *) malloc(sizeof(kfRotKeyLib3) * kframe[kframeList.count - 1].nrkeys);
  kframe[kframeList.count - 1].rkeys = (keyHeaderLib3 *) malloc(sizeof(keyHeaderLib3) * kframe[kframeList.count - 1].nrkeys);
 
  // read key header and immediately then read rotation and so on
  for( i = 0; i < kframe[kframeList.count - 1].nrkeys; i++)
  {

    // init values;
    kframe[kframeList.count - 1].rkeys[i].tension = 0.0;
    kframe[kframeList.count - 1].rkeys[i].continuity = 0.0;
    kframe[kframeList.count - 1].rkeys[i].bias = 0.0;
    kframe[kframeList.count - 1].rkeys[i].easeto = 0.0;
    kframe[kframeList.count - 1].rkeys[i].easefrom = 0.0;

    // key header
    // time
    fread(&kframe[kframeList.count - 1].rkeys[i].time, sizeof(unsigned long int), 1, inputFile);
    // used parameters 0 - 20 bytes (0 - 5 x float)
    fread(&kframe[kframeList.count - 1].rkeys[i].rflags, sizeof(unsigned short int), 1, inputFile);

    // select key header used parameters base from rflags
    if( kframe[kframeList.count - 1].rkeys[i].rflags & 1 )   fread(&kframe[kframeList.count - 1].rkeys[i].tension, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].rkeys[i].rflags & 2 )   fread(&kframe[kframeList.count - 1].rkeys[i].continuity, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].rkeys[i].rflags & 4 )   fread(&kframe[kframeList.count - 1].rkeys[i].bias, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].rkeys[i].rflags & 8 )   fread(&kframe[kframeList.count - 1].rkeys[i].easeto, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].rkeys[i].rflags & 16 )  fread(&kframe[kframeList.count - 1].rkeys[i].easefrom, sizeof(float), 1, inputFile);

    kframe[kframeList.count - 1].rkeys[i].tension = kframe[kframeList.count - 1].rkeys[i].tension*25 + 25;
    kframe[kframeList.count - 1].rkeys[i].continuity = kframe[kframeList.count - 1].rkeys[i].continuity*25 + 25;
    kframe[kframeList.count - 1].rkeys[i].bias = kframe[kframeList.count - 1].rkeys[i].bias*25 + 25;
    kframe[kframeList.count - 1].rkeys[i].easeto = kframe[kframeList.count - 1].rkeys[i].easeto*50;
    kframe[kframeList.count - 1].rkeys[i].easefrom = kframe[kframeList.count - 1].rkeys[i].easefrom*50;

    // point (rotation)
    fread(&kframe[kframeList.count - 1].rot[i].angle, sizeof(float), 1, inputFile);
    fread(&kframe[kframeList.count - 1].rot[i].x, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].rot[i].y, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].rot[i].z, sizeof(float), 1, inputFile);
  }
};

/*
 * This function read a all scales track tags for actual key frame
 */
void kframeDatabaseLib3::addScaleTT() {
  unsigned short int  readShort;
  unsigned long       readLong;
  unsigned int        i = 0;

  // first we have to read track header
  // uneccessary flags informations
  fread(&readShort, 2, 1, inputFile);

  // not currently used
  fread(&readLong, 4, 1, inputFile);

    // not currently used
  fread(&readLong, 4, 1, inputFile);

  // read number of keys in track
  fread(&kframe[kframeList.count - 1].nskeys, 4, 1, inputFile);
  	
  // alocate some memory for all the keys
  kframe[kframeList.count - 1].scale = (pointLib3 *) malloc(sizeof(pointLib3) * kframe[kframeList.count - 1].nskeys);
  kframe[kframeList.count - 1].skeys = (keyHeaderLib3 *) malloc(sizeof(keyHeaderLib3) * kframe[kframeList.count - 1].nskeys);

  // read key header and immediately then read scale and so on
  for(i = 0; i < kframe[kframeList.count - 1].nskeys; i++) {

    //init values
    kframe[kframeList.count - 1].skeys[i].tension = 0.0;
    kframe[kframeList.count - 1].skeys[i].continuity = 0.0;
    kframe[kframeList.count - 1].skeys[i].bias = 0.0;
    kframe[kframeList.count - 1].skeys[i].easeto = 0.0;
    kframe[kframeList.count - 1].skeys[i].easefrom = 0.0;

    // key header
    // time
    fread(&kframe[kframeList.count - 1].skeys[i].time, sizeof(unsigned long int), 1, inputFile);
    // used parameters 0 - 20 bytes (0 - 5 x float)
    fread(&kframe[kframeList.count - 1].skeys[i].rflags, sizeof(unsigned short int), 1, inputFile);

    // select key header used parameters base od rflags
    if( kframe[kframeList.count - 1].skeys[i].rflags & 1 )   fread(&kframe[kframeList.count - 1].skeys[i].tension, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].skeys[i].rflags & 2 )   fread(&kframe[kframeList.count - 1].skeys[i].continuity, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].skeys[i].rflags & 4 )   fread(&kframe[kframeList.count - 1].skeys[i].bias, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].skeys[i].rflags & 8 )   fread(&kframe[kframeList.count - 1].skeys[i].easeto, sizeof(float), 1, inputFile);
    if( kframe[kframeList.count - 1].skeys[i].rflags & 16 )  fread(&kframe[kframeList.count - 1].skeys[i].easefrom, sizeof(float), 1, inputFile);

    kframe[kframeList.count - 1].skeys[i].tension = kframe[kframeList.count - 1].skeys[i].tension*25 + 25;
    kframe[kframeList.count - 1].skeys[i].continuity = kframe[kframeList.count - 1].skeys[i].continuity*25 + 25;
    kframe[kframeList.count - 1].skeys[i].bias = kframe[kframeList.count - 1].skeys[i].bias*25 + 25;
    kframe[kframeList.count - 1].skeys[i].easeto = kframe[kframeList.count - 1].skeys[i].easeto*50;
    kframe[kframeList.count - 1].skeys[i].easefrom = kframe[kframeList.count - 1].skeys[i].easefrom*50;

    // point (scale)
    fread(&kframe[kframeList.count - 1].scale[i].x, sizeof(float), 1, inputFile);
    fread(&kframe[kframeList.count - 1].scale[i].y, sizeof(float), 1, inputFile);
		fread(&kframe[kframeList.count - 1].scale[i].z, sizeof(float), 1, inputFile);
  }
};

/*
 * Return kframe named "name". Search in all tracks
 */
kfmeshLib3 *kframeDatabaseLib3::dbGetKframe(char *name) {
  int i;
  char aux[KFRAME_PARENT_LIB_3DS_LEN];

  for( i = 0; i < kframeList.count; i++ ) {
    
    if( strstr( name, "$$$DUMMY") != NULL || kframe[i].instance[0] != '\0' ) { //strlen(name) >= KFRAME_NAME_LIB_3DS_LEN ) {
      strcpy(aux, kframe[i].name);
      strcat(aux, ".");
      strcat(aux,kframe[i].instance);
      if( strcmp(aux, name) == 0 )
        break;
    }
    else if( strcmp(kframe[i].name, name) == 0 )
      break;
  }

  if( i < kframeList.count )
    return(&kframe[i]);
  else
    return(NULL);

};

/*
 * Search in kframeList, where is kframe with same name, but different instances. This function
 * return indexth kframe's name instance
 */
kfmeshLib3 *kframeDatabaseLib3::dbGetKframeInstanceByIndex(char *name, int index) {
  int count = 0;

  for( int i = 0; i < kframeList.count; i++ ) {
    if( strcmp(kframe[i].name, name) == 0 ) {
      if( count == index )
        return(&kframe[i]);
      count++;
    }
  }
  
  return(dbGetKframe(name));
};

/*
 * This function return number of instances (+this kframe) belong to this kframe named name
 */
int kframeDatabaseLib3::dbGetMeshInstanceCount(char *name) {
  int count = 0;

  for( int i = 0; i < kframeList.count; i++ ) {
    if( strcmp(kframe[i].name, name) == 0 )
      count++;
  }

  return(count);
};

/*
 * material database constructor
 */
kframeDatabaseLib3::kframeDatabaseLib3(FILE *inpFile) {
  
 /* materialList.list = NULL;
  */
  nodeIdFound = 0;
  kframeList.count = 0;
  kframe = NULL;
  
  inputFile = inpFile;
};

/*
 * material database destructor
 */
kframeDatabaseLib3::~kframeDatabaseLib3() {
  int i;

  if( kframe != NULL ) 
  {
    for( i = 0; i < kframeList.count; i++)
    {
      if( kframe[i].pos != NULL ) { free(kframe[i].pos); kframe[i].pos = NULL;}
      if( kframe[i].pkeys != NULL ) {free(kframe[i].pkeys); kframe[i].pkeys = NULL;}
      if( kframe[i].rot != NULL ) { free(kframe[i].rot); kframe[i].rot= NULL;}
      if( kframe[i].rkeys != NULL ) {free(kframe[i].rkeys); kframe[i].rkeys= NULL;}
      if( kframe[i].scale != NULL ) {free(kframe[i].scale); kframe[i].scale= NULL;}
      if( kframe[i].skeys != NULL ) {free(kframe[i].skeys); kframe[i].skeys= NULL;}
    }
    free(kframe);
  }

  
};
