#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lib3ds.h"
#include "meshDatabase.h"
#include "materialDatabase.h"
#include "kframeDatabase.h"


#define CHUNK_ID_LEN      2
#define CHUNK_LENGHT_LEN  4

meshDatabaseLib3      *meshDatabase;
materialDatabaseLib3  *matDatabase;
kframeDatabaseLib3    *kframeDatabase;

/*
 * Return meshList from mesh database
 */
void lib3ds::getMeshList(meshListLib3 **meshList) {

  *meshList = meshDatabase->dbGetMeshList();
};

/*
 * Return mesh with name "name". This function search in meshList.
 */
void lib3ds::getMesh(char *name, meshLib3 **mesh) {
  int           i;
  meshListLib3  *meshList;

  meshList = meshDatabase->dbGetMeshList();

  for( i = 0; i < meshList->count; i++ )
    if( strcmp(meshList->list[i].name, name) == 0 )
      break;

  if( i < meshList->count )
    *mesh = meshDatabase->dbGetMesh(i);
  else
    mesh = NULL;    
};

/*
 * Return materialList from material database
 */
void lib3ds::getMaterialList(materialListLib3 **materialList) {

  *materialList = matDatabase->dbGetMaterialList();
};

/*
 * Return mesh with name "name". This function search in meshList.
 */
void lib3ds::getMaterial(char *name, materialLib3 **material) {
  int               i;
  materialListLib3  *materialList;

  materialList = matDatabase->dbGetMaterialList();

  for( i = 0; i < materialList->count; i++ )
    if( strcmp(materialList->list[i].name, name) == 0 )
      break;

  if( i < materialList->count )
    *material = matDatabase->dbGetMaterial(i);
  else
    material = NULL;    
};

/*
 * Return mesh's named "name" motion.
 */
void lib3ds::getKeyFrame(char *name, kfmeshLib3 **kframe) {
  *kframe = kframeDatabase->dbGetKframe(name);
}; 

void lib3ds::getKeyFrameInstanceByIndex(char *name, int index, kfmeshLib3 **kframe) {
  *kframe = kframeDatabase->dbGetKframeInstanceByIndex(name, index);
};

int lib3ds::getMeshInstanceCount(char *name) {
  return( kframeDatabase->dbGetMeshInstanceCount(name));
};

/*
 * This function create databaze from 3ds file in memory. This database is accesed
 * by user of this library
 */
int lib3ds::createModel(char *fileName)
{

  /*
   * General purpose of these variables is to move thru the 3ds file named fileName
   */
  unsigned short int    chunkId;      // chunk identifikator
  unsigned int          chunkLenght;  // lenght of actual chunk
  unsigned int		      chunkStart;   // start of actual chunk

  /*
   * Auxiliary variables
   */
  char                  name[MESH_NAME_LIB_3DS_LEN];
  int                   i;

  /*
   * Variables for working with inputFile
   */
  FILE                  *inputFile;
  long int              fileSize = 0;
  

  /*
   * Open 3ds file named fileName
   */
  if ( fileName != "" ) {
    if ( (inputFile = fopen(fileName, "rb")) == NULL )
      return(0);
  }
  else
    return(0);


  meshDatabase = new meshDatabaseLib3(inputFile);
  matDatabase = new materialDatabaseLib3(inputFile);
  kframeDatabase = new kframeDatabaseLib3(inputFile);

  // Get the 3ds inputFile named fileName size in bytes
  fseek(inputFile, 0, SEEK_END);
  fileSize = ftell(inputFile);
  fseek(inputFile, 0, SEEK_SET);
  
  /*
   * The Main algorithm to reach information from input 3ds file
   */

  // Go thru 3ds file and read all known chunks
  while( ftell(inputFile) < fileSize )
  {
    // next chunk started on actual position in inputFile
    chunkStart = ftell(inputFile);

    // read first two bytes, they determining chunk id
	  fread(&chunkId, CHUNK_ID_LEN, 1, inputFile);
    // read next four bytes, they determining chunk lenght
	  fread(&chunkLenght, CHUNK_LENGHT_LEN, 1, inputFile);

    // in base of read id get more information from actual chunk
    switch (chunkId)
    {
      // main chunk
      case 0x4d4d: break;    

      // 3ds editor chunk
      case 0x3d3d: break;

      // NAMED_OBJECT - mesh/camera/light object
      case 0x4000:
        i = 0;
        do fread(&name[i], 1, 1, inputFile);
        while(name[i++] != '\0' && i < MESH_NAME_LIB_3DS_LEN);
      break;

      /**************************************************************************************
       * N_TRI_OBJECT - mesh object *********************************************************
       **************************************************************************************/
      case 0x4100: meshDatabase->addMesh(name); break;
      
      //POINT_ARRAY 
      case 0x4110: meshDatabase->addPointArray(); break;

      //POINT_FLAG_ARRAY 
      case 0x4111: meshDatabase->addPointFlagArray(); break;

      //FACE_ARRAY 
      case 0x4120: meshDatabase->addFaceArray(); break;

      //MSH_MAT_GROUP 
      case 0x4130: meshDatabase->addMeshMatGroup(); break;

      //SMOOTH_GROUP 
      case 0x4150: meshDatabase->addSmoothArray(); break;

      //TEX_VERTS 
      case 0x4140: meshDatabase->addTextVertArray(); break;


      //MESH_MATRIX 
      case 0x4160: meshDatabase->addMeshMatrix(); break;


      /**************************************************************************************
      * MAT_ENTRY ***************************************************************************
      **************************************************************************************/

      case 0xAFFF: break;

      //MAT_NAME 
      case 0xA000: matDatabase->addMaterial(); break;

      //MAT_AMBIENT 
      case 0xA010: matDatabase->setColorType(ambient); break;

      //MAT_DIFFUSE
      case 0xA020: matDatabase->setColorType(diffuse); break;

      //MAT_SPECULAR
      case 0xA030: matDatabase->setColorType(specular); break;

      //MAT_SHININESS 
      case 0xA040: matDatabase->setPercentFor(shininess); break;

      //MAT_SHIN2PCT
      case 0xA041: matDatabase->setPercentFor(shin2pct); break;

      //MAT_TRANSPARENCY
      case 0xA050: matDatabase->setPercentFor(transparency); break;

      //MAT_USE_XPFALL 
      case 0xA240: matDatabase->setAtribute(usefall, 1); break;

      //MAT_XPFALL
      case 0xA052: matDatabase->setPercentFor(xpfall); break;

      //MAT_REFBLUR
      case 0xA053: matDatabase->setPercentFor(refblur); break;

      //MAT_SHADING 
      case 0xA100: matDatabase->setShading(); break;

      //MAT_USE_REFBLUR  
      case 0xA250: matDatabase->setAtribute(useblur, 1); break;

      //MAT_SELF_ILLUM 
      case 0xA080: matDatabase->setAtribute(selfillum, 1); break;

      //MAT_TWO_SIDE 
      case 0xA081: matDatabase->setAtribute(twosided, 1); break;

       //MAT_DECAL 
      case 0xA082: matDatabase->setAtribute(decal, 1); break;

      //MAT_ADDITIVE 
      case 0xA083: matDatabase->setAtribute(additive, 1); break;

      //MAT_SELF_ILPCT
      case 0xA084: matDatabase->setPercentFor(selfillpct); break;

      //MAT_WIRE 
      case 0xA085: matDatabase->setAtribute(usewire, 1); break;

      //MAT_FACEMAP 
      case 0xA088: matDatabase->setAtribute(facemap, 1); break;

      //MAT_PHONGSOFT 
      case 0xA08C: matDatabase->setAtribute(soften, 1); break;

      //MAT_WIREABS 
      case 0xA08E: matDatabase->setAtribute(usewireabs, 1); break;

      //MAT_WIRESIZE 
      case 0xA087: matDatabase->setWireSize(); break;

      /**************************************************************************************
      * COMMON_CHUNKS************************************************************************
      **************************************************************************************/
   
        //INT_PERCENTAGE 
      case 0x0030: matDatabase->readIntPercentage(); break;
		    
      //COLOR_24 
      case 0x0011: matDatabase->readColor24(); break;
      
      //LIN_COLOR_24 
      case 0x0012: matDatabase->readColor24(); break;
        
      /**************************************************************************************
      * TEXTURE *****************************************************************************
      **************************************************************************************/

      //MAT_TEXMAP 
      case 0xA200: matDatabase->setPercentFor(texture); break;

      //MAT_BUMPMAP 
      case 0xA230: matDatabase->setPercentFor(bump); break;

      //MAT_MAPNAME 
      case 0xA300: matDatabase->addMapName(); break;

      //MAT_MAP_TILING 
      case 0xA351: matDatabase->addMapTiling(); break;

      //MAT_MAP_TEXBLUR 
      case 0xA353: matDatabase->addMapTexBlur(); break;
    
      //MAT_MAP_USCALE 
      case 0xA354: matDatabase->addMapUscale(); break;
    
      //MAT_MAP_VSCALE
      case 0xA356: matDatabase->addMapVscale(); break;
 
      //MAT_MAP_UOFFSET
      case 0xA358: matDatabase->addMapUoffset(); break;

      //MAT_MAP_VOFFSET
      case 0xA35A: matDatabase->addMapVoffset(); break;

      //MAT_MAP_ANG
      case 0xA35C: matDatabase->addMapAngle(); break;


      /**************************************************************************************
      *Key frame data************************************************************************
      **************************************************************************************/

      //KFDATA 
 	    case 0xB000: break;

      //AMBIENT_NODE_TAG 
      case 0xB001: break;

	    //OBJECT_NODE_TAG 
 	    case 0xB002: break;

      //CAMERA_NODE_TAG 
      case 0xB003: break;

      //TARGET_NODE_TAG 
      case 0xB004: break;

      //LIGHT_NODE_TAG 
      case 0xB005: break;

      //SPOTLIGHT_NODE_TAG 
      case 0xB007: break;

      //L_TARGET_NODE_TAG 
      case 0xB006: break;


      //NODE_ID 
      case 0xB030: kframeDatabase->addKFandNode(1); break;

      //NODE_HDR 
      case 0xB010: kframeDatabase->addHeader(); break;

      //PIVOT 
      case 0xB013: kframeDatabase->addPivot(); break;

      //INSTANCE_NAME 
      case 0xB011: kframeDatabase->addInstanceName(); break;

      //BOUNDBOX 
	    case 0xB014: kframeDatabase->addBoundingBox(); break;

      //POS_TRACK_TAG 
      case 0xB020: kframeDatabase->addPositionTT(); break;

      //ROT_TRACK_TAG 
      case 0xB021: kframeDatabase->addRotationTT(); break;

      //SCL_TRACK_TAG 
      case 0xB022: kframeDatabase->addScaleTT(); break;
 
      default: fseek(inputFile, chunkLenght - (CHUNK_ID_LEN + CHUNK_LENGHT_LEN), SEEK_CUR); break;
    }
  }

  fclose(inputFile);

  return(1);
}

void lib3ds::releaseModel() {
  delete kframeDatabase;
  delete matDatabase;
  delete meshDatabase;
}
