#ifndef _lib3ds_h_
#define _lib3ds_h_

#define NAME_LIST_LIB_3DS_LEN     20
#define MESH_NAME_LIB_3DS_LEN     20
#define MATERIAL_NAME_LIB_3DS_LEN 17
#define LOCAL_MATRIX_SIZE         12
#define BITMAP_NAME_LIB_3DS_LEN   13
#define KFRAME_NAME_LIB_3DS_LEN   11
#define KFRAME_PARENT_LIB_3DS_LEN 22

struct pointLib3 {
  float x;
  float y;
  float z;
};

struct faceLib3 {
  unsigned short int v1;
  unsigned short int v2;
  unsigned short int v3;
  unsigned short int flag;
};

struct matLib3 {
  char               name[MATERIAL_NAME_LIB_3DS_LEN];
  unsigned short int nfaces;
  unsigned short int *faceindex;
}; 

struct textLib3 {
  float u;
  float v;
};

struct meshLib3 {
  char               name[MESH_NAME_LIB_3DS_LEN];
  unsigned short int nvertices;
  pointLib3          *vertexarray;
  unsigned short int nvflags;
  unsigned short int *vflagarray;
  unsigned short int ntextverts;
  textLib3           *textarray;
  float              locmatrix[LOCAL_MATRIX_SIZE];
  unsigned short int nfaces;
  faceLib3           *facearray;
  unsigned long int  *smootharray;
  unsigned short int nmats;
  matLib3            *matarray;
};




typedef struct {
  char name[NAME_LIST_LIB_3DS_LEN];
} nameListLib3;

struct materialListLib3  {
  nameListLib3 *list;
  int count;
};

struct meshListLib3  {
  nameListLib3 *list;
  int count;
};

struct kframeListLib3  {
  int count;
};


struct colorLib3 {
  float r;
  float g;
  float b;
};

enum shadeTypeLib3 {
  Wire=    0,
  Flat=    1,
  Gouraud= 2,
  Phong=   3,
  Metal=   4
};

enum tileTypeLib3 {
  Tile=   1,
  Decal=  2,
  Both=   3
};

enum filterTypeLib3 {
  Pyramidal,
  SummedArea
};

enum tintTypeLib3 {
  RGB,
  Alpha,
  RGBLumaTint,
  AlphaTint,
  RGBTint
};

struct bitmapLib3 {
  char             name[BITMAP_NAME_LIB_3DS_LEN];
  float            percent;
  tileTypeLib3     tiling;
  char             ignorealpha;
  filterTypeLib3   filter;
  float            blur;
  char             mirror;
  char             negative;
  float            uscale;
  float            vscale;
  float            uoffset;
  float            voffset;
  float            rotation;
  tintTypeLib3     source;
};

struct mapMaskLib3 {
  bitmapLib3 map;
  bitmapLib3 mask;
};

struct materialLib3 {
  char             name[MATERIAL_NAME_LIB_3DS_LEN];
  colorLib3        ambient;
  colorLib3        diffuse;
  colorLib3        specular;
  float            shininess;
  float            shinstrength;
  float            blur;
  float            transparency;
  float            transfalloff;
  float            selfillumpct;
  float            wiresize;
  shadeTypeLib3    shading;
  char             useblur;
  char             usefall;
  char             twosided;
  char             selfillum;
  char             additive;
  char             usewire;
  char             usewireabs;
  char             facemap;
  char             soften;
  char             decal;
  mapMaskLib3      texture;
  mapMaskLib3      bump;
};


struct keyHeaderLib3 {
  unsigned long int  time;
  unsigned short int rflags;
  float              tension;
  float              continuity;
  float              bias;
  float              easeto;
  float              easefrom;
};

struct kfRotKeyLib3 {
  float angle; 
  float x; 
  float y;
  float z;
};

struct kfmeshLib3 {
  unsigned short int  nodeid;
  char                name[KFRAME_NAME_LIB_3DS_LEN];
  char                parent[KFRAME_PARENT_LIB_3DS_LEN];
  unsigned short int  sparent;
  unsigned short int  flags1;
  unsigned short int  flags2;
  pointLib3           pivot; 
  char                instance[KFRAME_NAME_LIB_3DS_LEN];
  pointLib3           boundmin; 
  pointLib3           boundmax; 
  unsigned long int   npkeys; 
  short int           npflag; 
  keyHeaderLib3       *pkeys; 
  pointLib3           *pos; 
  unsigned long int   nrkeys; 
  short int           nrflag; 
  keyHeaderLib3       *rkeys; 
  kfRotKeyLib3        *rot; 
  unsigned long int   nskeys; 
  short int           nsflag; 
  keyHeaderLib3       *skeys; 
  pointLib3           *scale; 
};



class lib3ds {
  
public:

  void getMeshList(meshListLib3 **);
  void getMesh(char *name, meshLib3 **);
  
  void getMaterialList(materialListLib3 **);
  void getMaterial(char *name, materialLib3 **);
  
  void getKeyFrame(char *name, kfmeshLib3 **); 
  void getKeyFrameInstanceByIndex(char *name, int index, kfmeshLib3 **);
  int  getMeshInstanceCount(char *);

  // TODO:
  // - function to extract informations such as vertices, normals, faces with 
  //   animation aplicated on it.
  // - function to extract all information from mesh realated to one metarial from 
  //   material collection used with one mesh.
  // - more library testing

  // SOMETIME IN FAR FUTURE:
  // - add light support
  // - add camera support
  // - add more texture types support (now classic texture and bump map is supported)
  // - and so on
    
  int createModel(char *name);
  void releaseModel();

  lib3ds() {};
  ~lib3ds() {};
};

#endif
