#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "materialDatabase.h"

/*
 * This function set actual color type to going to process
 */
void materialDatabaseLib3::setColorType(col_type color) {
  color_type = color;  
};

/*
 * This function set actual atribute to read from percent chunk
 */
void materialDatabaseLib3::setPercentFor(per_type percent) {
  percent_type = percent;
};

/*
 * This function tell us that some of bellow written parameters was appear in 3ds file for
 * actual material
 */
void materialDatabaseLib3::setAtribute(attr_type attribute, int value) {
  
  switch( attribute) {
  
    case useblur    : material[materialList.count - 1].useblur = value; break;
    case usefall    : material[materialList.count - 1].usefall = value; break;
    case selfillum  : material[materialList.count - 1].selfillum = value; break;
    case twosided   : material[materialList.count - 1].twosided = value; break;
    case decal      : material[materialList.count - 1].decal = value; break;
    case additive   : material[materialList.count - 1].additive = value; break;
    case usewire    : material[materialList.count - 1].usewire = value; break;
    case facemap    : material[materialList.count - 1].facemap = value; break;
    case soften     : material[materialList.count - 1].soften = value; break;
    case usewireabs : material[materialList.count - 1].usewireabs = value; break;
    default : break;
  }
};

/*
 * Get information about material shading method
 */
void materialDatabaseLib3::setShading() {
  fread(&material[materialList.count - 1].shading, sizeof(short int), 1, inputFile);
};

/*
 * Get information about material shading method
 */
void materialDatabaseLib3::setWireSize() {
  fread(&material[materialList.count - 1].wiresize, sizeof(float), 1, inputFile);
};

/*
 * This function read value from 3ds file bases on percent_type attr
 * add this value to right material attribute
 */
void materialDatabaseLib3::readIntPercentage() {
  short int readsShort;
  float     readFloat;

  fread(&readsShort, sizeof(short int), 1, inputFile);
  readFloat = (float) (readsShort / 100.0);
        
  switch(percent_type)
  {
    case shininess: material[materialList.count - 1].shininess =          readFloat; break;
    case shin2pct: material[materialList.count - 1].shinstrength =        readFloat; break;
    case transparency: material[materialList.count - 1].transparency =    readFloat; break;
    case xpfall: material[materialList.count - 1].transfalloff =          readFloat; break;
    case refblur: material[materialList.count - 1].blur =                 readFloat; break;
    case selfillpct: material[materialList.count - 1].selfillumpct =      readFloat; break;

    case texture: material[materialList.count - 1].texture.map.percent =  readFloat; break;
    case bump: material[materialList.count - 1].bump.map.percent =        readFloat; break;
    default: break;
  }

};

/*
 * This function read value from 3ds file based on color_type attr
 * add this value to right material attribute
 */
void materialDatabaseLib3::readColor24() {
  unsigned char         ucr;
  unsigned char         ucg;
  unsigned char         ucb;
  
  fread(&ucr, sizeof(unsigned char), 1, inputFile);
  fread(&ucg, sizeof(unsigned char), 1, inputFile);
  fread(&ucb, sizeof(unsigned char), 1, inputFile);

  switch( color_type )
  {
    case ambient:
      material[materialList.count - 1].ambient.r = (float) (ucr / 256.0);
      material[materialList.count - 1].ambient.g = (float) (ucg / 256.0);
      material[materialList.count - 1].ambient.b = (float) (ucb / 256.0);
    break;

    case diffuse:
      material[materialList.count - 1].diffuse.r = (float) (ucr / 256.0);
      material[materialList.count - 1].diffuse.g = (float) (ucg / 256.0);
      material[materialList.count - 1].diffuse.b = (float) (ucb / 256.0);
    break;

    case specular:
      material[materialList.count - 1].specular.r = (float) (ucr / 256.0);
      material[materialList.count - 1].specular.g = (float) (ucg / 256.0);
      material[materialList.count - 1].specular.b = (float) (ucb / 256.0);
    break;
    default: break;
  }
};


/*
 * This function read value from 3ds file based on color_type attr
 * add this value to right material attribute
 */
void materialDatabaseLib3::readLinearColor24() {
  unsigned char         ucr;
  unsigned char         ucg;
  unsigned char         ucb;
  
  fread(&ucr, sizeof(unsigned char), 1, inputFile);
  fread(&ucg, sizeof(unsigned char), 1, inputFile);
  fread(&ucb, sizeof(unsigned char), 1, inputFile);
            

  switch( color_type )
  {
    case ambient:
      material[materialList.count - 1].ambient.r = (float) (ucr / 256.0);
      material[materialList.count - 1].ambient.g = (float) (ucg / 256.0);
      material[materialList.count - 1].ambient.b = (float) (ucb / 256.0);
    break;

    case diffuse:
      material[materialList.count - 1].diffuse.r = (float) (ucr / 256.0);
      material[materialList.count - 1].diffuse.g = (float) (ucg / 256.0);
      material[materialList.count - 1].diffuse.b = (float) (ucb / 256.0);
    break;

    case specular:
      material[materialList.count - 1].specular.r = (float) (ucr / 256.0);
      material[materialList.count - 1].specular.g = (float) (ucg / 256.0);
      material[materialList.count - 1].specular.b = (float) (ucb / 256.0);
    break;
    default: break;
  }
};

/*
 * This function init actual material's textures map name
 */
void materialDatabaseLib3::addMapName() {
  int i=0;
  char name[BITMAP_NAME_LIB_3DS_LEN];

  do fread(&name[i], sizeof(unsigned char), 1, inputFile);
  while(name[i++] != '\0' && i < BITMAP_NAME_LIB_3DS_LEN);  

  switch(percent_type)
  {
    case texture: strcpy(material[materialList.count - 1].texture.map.name, name); break;
    case bump   : strcpy(material[materialList.count - 1].bump.map.name, name);    break;
    default: break;
  }
};

/*
 * This function init actual material's textures map tiling ethod
 */
void materialDatabaseLib3::addMapTiling() {
  unsigned short int readShort; 
  fread(&readShort, sizeof(unsigned short int), 1, inputFile);
  
  switch(percent_type)
  {
    case texture: 
      material[materialList.count - 1].texture.map.tiling = (tileTypeLib3) (abs((readShort & 0x1) * 3 - (readShort & 0x10)) );
      material[materialList.count - 1].texture.map.mirror = readShort & 0x2; //
      material[materialList.count - 1].texture.map.negative = readShort & 0x8; //
      material[materialList.count - 1].texture.map.filter = (filterTypeLib3) (readShort & 0x20);
      material[materialList.count - 1].texture.map.ignorealpha = (filterTypeLib3) (readShort & 0x100);
    break;
            
    case bump: 
      material[materialList.count - 1].texture.map.tiling = (tileTypeLib3) (abs((readShort & 0x1) * 3 - (readShort & 0x10)) );
      material[materialList.count - 1].texture.map.mirror = readShort & 0x2; //
      material[materialList.count - 1].texture.map.negative = readShort & 0x8; //
      material[materialList.count - 1].texture.map.filter = (filterTypeLib3) (readShort & 0x20);
      material[materialList.count - 1].texture.map.ignorealpha = (filterTypeLib3) (readShort & 0x100);
    break;
    default: break;
  }  
}

/*
 * This function read one texture parameter and wirite it to the material database
 */
void materialDatabaseLib3::addMapTexBlur(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.blur, sizeof(float), 1, inputFile); break;
    case bump   : fread(&material[materialList.count - 1].bump.map.blur, sizeof(float), 1, inputFile);    break;
    default: break;
  }
};

void materialDatabaseLib3::addMapUscale(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.uscale, sizeof(float), 1, inputFile); break;
    case bump   : fread(&material[materialList.count - 1].bump.map.uscale, sizeof(float), 1, inputFile);    break;
    default: break;
  }
};

void materialDatabaseLib3::addMapVscale(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.vscale, sizeof(float), 1, inputFile); break;
    case bump   : fread(&material[materialList.count - 1].bump.map.vscale, sizeof(float), 1, inputFile);    break;
    default: break;
  }
};

void materialDatabaseLib3::addMapUoffset(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.uoffset, sizeof(float), 1, inputFile);  break;
    case bump   : fread(&material[materialList.count - 1].bump.map.uoffset, sizeof(float), 1, inputFile);     break;
    default: break;
  }
};

void materialDatabaseLib3::addMapVoffset(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.voffset, sizeof(float), 1, inputFile);  break;
    case bump   : fread(&material[materialList.count - 1].bump.map.voffset, sizeof(float), 1, inputFile);     break;
    default: break;
  }
};

void materialDatabaseLib3::addMapAngle(){
  switch(percent_type)
  {
    case texture: fread(&material[materialList.count - 1].texture.map.rotation, sizeof(float), 1, inputFile); break;
    case bump   : fread(&material[materialList.count - 1].bump.map.rotation, sizeof(float), 1, inputFile);    break;
    default: break;
 }
};


/*
 * Add material to database and add this material's name to materialList
 * Init material default's values too
 */
void materialDatabaseLib3::addMaterial() {
  int i = 0;
  char name[MATERIAL_NAME_LIB_3DS_LEN];

  // get material name from 3ds file
  do fread(&name[i], sizeof(unsigned char), 1, inputFile);
  while(name[i++] != '\0' && i < MATERIAL_NAME_LIB_3DS_LEN);

  // add material to material list
  materialList.count += 1;
  materialList.list = (nameListLib3 *) realloc(materialList.list, sizeof(nameListLib3) * materialList.count);
  strcpy(materialList.list[materialList.count - 1].name, name);
  
  // alocate some memory for new material
  material = (materialLib3 *) realloc(material, sizeof(materialLib3) * materialList.count);

  strcpy(material[materialList.count - 1].name, name);

  // set default values for each new material
  material[materialList.count - 1].shininess =    1;
  material[materialList.count - 1].shinstrength = 0;
  material[materialList.count - 1].blur =         0;
  material[materialList.count - 1].transparency = 0;
  material[materialList.count - 1].transfalloff = 0;
  material[materialList.count - 1].selfillumpct = 0;
  material[materialList.count - 1].wiresize =     1;
  material[materialList.count - 1].shading =      Flat;
  material[materialList.count - 1].usefall =      0;
  material[materialList.count - 1].useblur =      0;
  material[materialList.count - 1].selfillum =    0;
  material[materialList.count - 1].twosided =     0;
  material[materialList.count - 1].decal =        0;
  material[materialList.count - 1].additive =     0;
  material[materialList.count - 1].usewire =      0;
  material[materialList.count - 1].facemap =      0;
  material[materialList.count - 1].soften =       0;
  material[materialList.count - 1].usewireabs =   0;

  material[materialList.count - 1].texture.map.name[0] =      '\0';
  material[materialList.count - 1].texture.map.uoffset =      0.0;
  material[materialList.count - 1].texture.map.voffset =      0.0;
  material[materialList.count - 1].texture.map.uscale =       1.0;
  material[materialList.count - 1].texture.map.vscale =       1.0;
  material[materialList.count - 1].texture.map.rotation =     0.0;
  material[materialList.count - 1].texture.map.negative =     0;
  material[materialList.count - 1].texture.map.mirror =       0;
  material[materialList.count - 1].texture.map.blur =         0.0;
  material[materialList.count - 1].texture.map.filter =       Pyramidal;
  material[materialList.count - 1].texture.map.ignorealpha =  0;
  material[materialList.count - 1].texture.map.tiling =       Tile;
  material[materialList.count - 1].texture.map.percent =      0.0;

  material[materialList.count - 1].bump.map.name[0] =         '\0';
  material[materialList.count - 1].bump.map.uoffset =         0.0;
  material[materialList.count - 1].bump.map.voffset =         0.0;

  material[materialList.count - 1].bump.map.uscale =          1.0;
  material[materialList.count - 1].bump.map.vscale =          1.0;
  material[materialList.count - 1].bump.map.rotation =        0.0;
  material[materialList.count - 1].bump.map.negative =        0;
  material[materialList.count - 1].bump.map.mirror =          0;
  material[materialList.count - 1].bump.map.blur =            0.0;
  material[materialList.count - 1].bump.map.filter =          Pyramidal;
  material[materialList.count - 1].bump.map.ignorealpha =     0;
  material[materialList.count - 1].bump.map.tiling =          Tile;
  material[materialList.count - 1].bump.map.percent =         0.0;
}

/*
 * Get pointer to the materialList from database
 */
materialListLib3 *materialDatabaseLib3::dbGetMaterialList() {
  return(&materialList);
}

/*
 * Get pointer to teh meaterial from database
 */
materialLib3 *materialDatabaseLib3::dbGetMaterial(int i) {
  return(&material[i]);
}


/*
 * material database constructor
 */
materialDatabaseLib3::materialDatabaseLib3(FILE *inpFile) {
  
  materialList.list = NULL;
  materialList.count = 0;
  
  material = NULL;
  
  inputFile = inpFile;
};

/*
 * material database destructor
 */
materialDatabaseLib3::~materialDatabaseLib3() {

  if( materialList.list != NULL ) free(materialList.list);
  if( material != NULL ) free(material);
};
